==> ideaIC-2019.2.1-sources.jar: com/android/tools/idea/res/PsiDataBindingResourceItem.kt <==
/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.res

import com.android.SdkConstants
import com.android.ide.common.resources.DataBindingResourceType
import com.intellij.openapi.util.text.StringUtil
import com.intellij.psi.xml.XmlTag

/**
 * Represents a resource item that is related to data binding, extracted from a layout file.
 */
data class PsiDataBindingResourceItem private constructor(
  val name: String,
  val type: DataBindingResourceType,
  val xmlTag: XmlTag,
  val source: PsiResourceFile,
  private val data: Map<String, String>
) {
  constructor(name: String,
              type: DataBindingResourceType,
              xmlTag: XmlTag,
              source: PsiResourceFile) : this(
    name = name,
    type = type,
    xmlTag = xmlTag,
    source = source,
    data = type.attributes.associate { attr ->
      attr to StringUtil.unescapeXml(xmlTag.getAttributeValue(attr))
    }
  )

  /**
   * Use [typeDeclaration] to get the type instead of this method.
   */
  fun getExtra(name: String): String? {
    return data[name]
  }

  /**
   * Same as sanitized output of [getExtra(SdkConstants.ATTR_TYPE)](getExtra).
   */
  val typeDeclaration: String?
    get() {
      val type = getExtra(SdkConstants.ATTR_TYPE)
      return type?.replace('$', '.')
    }
}

==> ideaIC-2019.3.5-sources.jar: com/android/tools/idea/res/PsiDataBindingResourceItem.kt <==
/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.res

import com.android.SdkConstants
import com.android.ide.common.resources.DataBindingResourceType
import com.android.tools.idea.databinding.DataBindingUtil
import com.intellij.openapi.util.text.StringUtil
import com.intellij.psi.xml.XmlTag

/**
 * Represents a resource item that is related to data binding, extracted from a layout file.
 */
data class PsiDataBindingResourceItem(
  val name: String,
  val type: DataBindingResourceType,
  val xmlTag: XmlTag,
  val source: PsiResourceFile) {

  /**
   * If you are planning to call this with [SdkConstants.ATTR_TYPE], use [typeDeclaration] instead.
   */
  fun getExtra(name: String): String? {
    return type.attributes
      .find { it == name }
      ?.let { attr -> StringUtil.unescapeXml(xmlTag.getAttributeValue(attr)) }
  }

  /**
   * Same as sanitized output of [getExtra(SdkConstants.ATTR_TYPE)](getExtra).
   */
  val typeDeclaration: String?
    get() = getExtra(SdkConstants.ATTR_TYPE)?.replace('$', '.')
}
